/****************************************************************************
 * Projekt - HMI-Unit
 * (c) 2021 softwareentwicklung-als-prozess.de
 * Dokumentation s. Blog
 * Funktionalitaet:
 * Alle Elemente einer Mensch Maschine Schnittstelle auf einem Board als unabhaengige Einheit
 * auf einem Board. Gesteuert ueber eine serielle Schnittstelle.
 ***************************************************************************/
/* --- Compiler Includes ---------- */
#include <arduino.h>
#include <Keypad.h>

/* --- Includes ---------- */
#include "buzkeys.h"

/* --- defines ----------*/
#define BUZZPIN 12

/* --- Variablen ---------- */
// buzzer
unsigned long buzstart  = 0;
unsigned long buzperiod = 0;
bool    buzinterval = false;
uint8_t buzivcnt    = 0;
uint8_t buzcnt      = 0;
// keypad
bool pushkeys = false;
char hexaKeys[2][3] =  //define the symbols on the buttons of the keypads
{
  {'B','R','U'},
  {'A','D','L'}
};
byte rowPins[2] = {A4, A5};     //connect to the row pinouts of the keypad
byte colPins[3] = {A1, A2, A3}; //connect to the column pinouts of the keypad
Keypad kpd = Keypad( makeKeymap(hexaKeys), rowPins, colPins, 2, 3); 

/* --- Funktionen ---------- */

/*
 * initialisert buzzer
 */
void buzkeys_Init(void)
{
  pinMode(BUZZPIN, OUTPUT);
  digitalWrite(BUZZPIN, LOW);
}

/*
 * schaltet den buzzer ein
 */
void buzkeys_SetBuzz(void)
{
  if (buzinterval == true)
  {
    buzinterval = false;
    buzstart    = 0;
    buzperiod   = 0;
  }
  digitalWrite(BUZZPIN, HIGH);
}

/*
 * schaltet den buzzer aus
 */
void buzkeys_ClearBuzz(void)
{
  buzstart    = 0;
  buzinterval = false;
  buzperiod   = 0;
  digitalWrite(BUZZPIN, LOW);
}

/*
 * schaltet die Intervallhupe ein
 */
void buzkeys_SetPeriod(unsigned long period)
{
  buzstart    = millis();
  buzinterval = true;
  buzperiod   = period;
}

/*
 * setzt die anzahl der hupintervalle 
 */
void buzkeys_SetIntervallCount(uint8_t cnt)
{
  buzcnt = cnt;
}
 
/*
 * macht die intervallhupe
 * nach mehr als 50 Tagen gibt es einen ueberlauf in millis()
 */
void buzkeys_CheckBuzz(void)
{
  if (buzinterval == true)
  {
    if((millis() - buzstart) >= buzperiod)
    {
      buzstart = millis();
      digitalWrite(BUZZPIN, !digitalRead(BUZZPIN));
      buzivcnt += 1;
      if (buzivcnt == (buzcnt * 2))
      {
        buzkeys_ClearBuzz();
        buzivcnt = 0;
        buzcnt   = 0;
      }
    }
  }
}

void buzkeys_SetPushKey(void)
{
  pushkeys = true;  
}

void buzkeys_ClearPushKey(void)
{
  pushkeys = false;
}

/*
 * prueft alle keys
 */
void buzkeys_CheckKeys(void)
{
  int    i = 0;
  bool fnd = false;
  
  // Fills kpd.key[ ] array with up-to 10 active keys.
  // Returns true if there are ANY active keys.
  if (kpd.getKeys())
  {
    for (i = 0; i < LIST_MAX; i++)   // Scan the whole key list.
    {
      if ( kpd.key[i].stateChanged )   // Only find keys that have changed state.
      {
        switch (kpd.key[i].kstate) 
        {
          case PRESSED:
            Serial1.write(kpd.key[i].kchar);
            fnd = true;
            break;
          case HOLD:
            break;
          case RELEASED:
            break;
          case IDLE:
            break;
        }
      }
    }
    if (fnd == true)
    {
      Serial1.write("\n");
      fnd = false;
    }
  }
}

void buzkeys_CheckPushKeys(void)
{
  if (pushkeys == true)
  {
    buzkeys_CheckKeys();
  }
}
