/****************************************************************************
 * Projekt - HMI-Unit
 * (c) 2021 softwareentwicklung-als-prozess.de
 * Dokumentation s. Blog
 * Funktionalitaet:
 * Alle Elemente einer Mensch Maschine Schnittstelle auf einem Board als unabhaengige Einheit
 * auf einem Board. Gesteuert ueber eine serielle Schnittstelle.
 ***************************************************************************/
/* --- Compiler Includes ---------- */
#include <arduino.h>
#include <SPI.h>
#include <Adafruit_GFX.h>
#include <Adafruit_PCD8544.h>

/* --- Includes ---------- */
#include "display.h"

/* --- defines ----------*/

/* --- Variablen ---------- */
// Software SPI (slower updates, more flexible pin options):
// pin 7 - Serial clock out (SCLK)
// pin 6 - Serial data out (DIN)
// pin 5 - Data/Command select (D/C)
// pin 4 - LCD chip select (CS)
// pin 3 - LCD reset (RST)
Adafruit_PCD8544 display = Adafruit_PCD8544(7, 6, 5, 4, 3);

/* --- Funktionen ---------- */

/*
 * initialisert display
 */
void display_Init(void)
{
  display.begin();
  display.setTextSize(1);
  display.setTextColor(BLACK);
  display.setCursor(0,0);
  display.setContrast(50);
  display.display();         // show splashscreen
  delay(2000);
  display.clearDisplay();
  display.display();
}

void display_Clear(void)
{
  display.setCursor(0,0);
  display.clearDisplay();   // clears the screen and buffer
  display.display();
}

void display_TextSize(uint8_t siz)
{
  display.setTextSize(siz);
}

void display_WriteText(String txt)
{
  display.println(txt);
  display.display();
}

void display_CursorPosition(uint8_t xpos, uint8_t ypos)
{
  display.setCursor(xpos,ypos); // x -> nach rechts, y nach unten, -> Pixelposition!
  // für Text 6 bits breit, 8 bits hoch, jeweils mal vergoesserungsfaktor 
  // fontsize laut bibliothek 5 breit 7 hoch plus je 1 bit zwischenraum
}

void display_DrawLine(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1)
{
  display.drawLine(x0, y0, x1, y1, BLACK);
  display.display();
}

void display_DrawPixel(uint8_t x, uint8_t y)
{
  display.drawPixel(x, y, BLACK);
  display.display();
}
