/****************************************************************************
 * Projekt - HMI-Unit
 * (c) 2021 softwareentwicklung-als-prozess.de
 * Dokumentation s. Blog
 * Funktionalitaet:
 * Alle Elemente einer Mensch Maschine Schnittstelle auf einem Board als unabhaengige Einheit
 * auf einem Board. Gesteuert ueber eine serielle Schnittstelle.
 ***************************************************************************/
/* --- Compiler Includes ---------- */
#include <arduino.h>

/* --- Includes ---------- */
#include "leds.h"

/* --- defines ----------*/
#define BLUEPIN 11
#define GREENPIN 10
#define REDPIN 9
#define BACKPIN 13

/* --- Variablen ---------- */
// alles fuers blinken
unsigned long greenstart  = 0;
bool          greenblink  = false;
unsigned long greenperiod = 0;
unsigned long bluestart   = 0;
bool          blueblink   = false;
unsigned long blueperiod  = 0;
unsigned long redstart    = 0;
bool          redblink    = false;
unsigned long redperiod   = 0;
unsigned long backstart   = 0;
bool          backblink   = false;
unsigned long backperiod  = 0;

/* --- Funktionen ---------- */
/*
 * Initialisiert die LED's
 * und setzt alle auf High, da der 2803 als Treiber die Signale invertiert
 */
void leds_Init(void)
{
  pinMode(REDPIN, OUTPUT);
  digitalWrite(REDPIN, HIGH);
  pinMode(GREENPIN, OUTPUT);
  digitalWrite(GREENPIN, HIGH);
  pinMode(BLUEPIN, OUTPUT);
  digitalWrite(BLUEPIN, HIGH);
  pinMode(BACKPIN, OUTPUT);
  digitalWrite(BACKPIN, HIGH);
}

/*
 * schaltet die blaue led ein
 */
void leds_SetBlue(void)
{
  if (blueblink == true)
  {
    blueblink  = false;
    bluestart  = 0;
    blueperiod = 0;
  }
  digitalWrite(BLUEPIN, LOW);
}

/*
 * schaltet die blaue led aus
 */
void leds_ClearBlue(void)
{
  bluestart  = 0;
  blueblink  = false;
  digitalWrite(BLUEPIN, HIGH);
}

/*
 * schaltet das blinken der blauen led ein
 */
void leds_SetBlinkBlue(unsigned long period)
{
  bluestart  = millis();
  blueblink  = true;
  blueperiod = period;
}
 
/*
 * startet das dimmen der blauen led
 */
void leds_DimmBlue(int wrt)
{
  if (blueblink == true)
  {
    blueblink  = false;
    bluestart  = 0;
    blueperiod = 0;
  }
  analogWrite(BLUEPIN,wrt);
}

/*
 * schaltet die gruene led ein
 */
void leds_SetGreen(void)
{
  if (greenblink == true)
  {
    greenblink  = false;
    greenstart  = 0;
    greenperiod = 0;
  }
  digitalWrite(GREENPIN, LOW);
}

/*
 * schaltet die gruen led aus
 */
void leds_ClearGreen(void)
{
  greenstart  = 0;
  greenblink  = false;
  digitalWrite(GREENPIN, HIGH);
}

/*
 * schaltet das blinken der gruenen led ein
 */
void leds_SetBlinkGreen(unsigned long period)
{
  greenstart  = millis();
  greenblink  = true;
  greenperiod = period;
}

/*
 * startet das dimmen der gruenen led
 */
void leds_DimmGreen(int wrt)
{
  if (greenblink == true)
  {
    greenblink  = false;
    greenstart  = 0;
    greenperiod = 0;
  }
  analogWrite(GREENPIN,wrt);
}

/*
 * schaltet die rote led ein
 */
void leds_SetRed(void)
{
  if (redblink == true)
  {
    redblink  = false;
    redstart  = 0;
    redperiod = 0;
  }
  digitalWrite(REDPIN, LOW);
}

/*
 * schaltet die rote led aus
 */
void leds_ClearRed(void)
{
  redstart  = 0;
  redblink  = false;
  digitalWrite(REDPIN, HIGH);
}

/*
 * schaltet das blinken der roten led ein
 */
void leds_SetBlinkRed(unsigned long period)
{
  redstart  = millis();
  redblink  = true;
  redperiod = period;
}

/*
 * startet das dimmen der roten led
 */
void leds_DimmRed(int wrt)
{
  if (redblink == true)
  {
    redblink  = false;
    redstart  = 0;
    redperiod = 0;
  }
  analogWrite(REDPIN,wrt);
}

/*
 * schaltet die backlight led ein
 */
void leds_SetBacklight(void)
{
  if (backblink == true)
  {
    backblink  = false;
    backstart  = 0;
    backperiod = 0;
  }
  digitalWrite(BACKPIN, LOW);
}

/*
 * schaltet die backlight led aus
 */
void leds_ClearBacklight(void)
{
  backstart  = 0;
  backblink  = false;
  digitalWrite(BACKPIN, HIGH);
}

/*
 * schaltet das blinken der backlight led ein
 */
void leds_SetBlinkBacklight(unsigned long period)
{
  backstart  = millis();
  backblink  = true;
  backperiod = period;
}

/*
 * startet das dimmen der Backlight led
 */
void leds_DimmBacklight(int wrt)
{
  if (backblink == true)
  {
    backblink  = false;
    backstart  = 0;
    backperiod = 0;
  }
  analogWrite(BACKPIN,wrt);
}

/*
 * macht die blinkerei
 * nach mehr als 50 Tagen gibt es einen ueberlauf in millis()
 */
void leds_CheckBlink(void)
{
  if (greenblink == true)
  {
    if((millis() - greenstart) >= greenperiod)
    {
      greenstart = millis();
      digitalWrite(GREENPIN, !digitalRead(GREENPIN));
    }
  }
  if (blueblink == true)
  {
    if((millis() - bluestart) >= blueperiod)
    {
      bluestart = millis();
      digitalWrite(BLUEPIN, !digitalRead(BLUEPIN));
    }
  }
  if (redblink == true)
  {
    if((millis() - redstart) >= redperiod)
    {
      redstart = millis();
      digitalWrite(REDPIN, !digitalRead(REDPIN));
    }
  }
  if (backblink == true)
  {
    if((millis() - backstart) >= backperiod)
    {
      backstart = millis();
      digitalWrite(BACKPIN, !digitalRead(BACKPIN));
    }
  }
}
