/****************************************************************************
 * Projekt - HMI-Unit
 * (c) 2021 softwareentwicklung-als-prozess.de
 * Dokumentation s. Blog
 * Funktionalitaet:
 * Alle Elemente einer Mensch Maschine Schnittstelle auf einem Board als unabhaengige Einheit
 * auf einem Board. Gesteuert ueber eine serielle Schnittstelle.
 ***************************************************************************/
/* --- Compiler Includes ---------- */
#include <arduino.h>

/* --- Includes ---------- */
#include "serial.h"
#include "leds.h"
#include "buzkeys.h"
#include "display.h"

/* interne typedefs ----------- */
typedef enum
{
  red,
  green,
  blue,
  backlight  
} ledcolor;

/* --- Interne Prototypen ---------- */
static void    ser_SendHmiStatus(void);
static void    CheckLedCommand(void);
static void    CheckLedSubCommand(ledcolor led, uint8_t startpos);
static uint8_t CheckPosition(uint32_t vl);
static void    CheckBuzCommand(void);
static void    CheckKeyCommand(void);
static void    CheckDisplayCommand(void);

/* --- Variablen ---------- */
String inputString    = "";
String outputString   = "";
bool   stringComplete = false;

/* --- Funktionen ---------- */
/*
 * initialisiert die serielle Schnittstelle
 */
void ser_Init(void)
{
  Serial1.begin(115200, SERIAL_8N1);
  inputString.reserve(120);
  outputString.reserve(120);
}

/*
 * gibt die Kennung des Programms aus
 */
void ser_StartUp(void)
{
  Serial1.write("HMI Unit V0.1\n");
}

/*
 * loescht den eingelesenen String
 */
void ser_ClearInputString(void)
{
    inputString = "";
    stringComplete = false;
}

/*
 * Parser fuer den eingelesenen String
 */
void ser_CheckInput(void)
{
  String  keyword;

  if (stringComplete == true)
  {
    outputString = inputString;
    inputString.toUpperCase();
    keyword = inputString.substring(0,3);
    
    if (keyword.compareTo("LED") == 0)
    {
      CheckLedCommand();
    }
    else
      if (keyword.compareTo("STS") == 0)
      {
        ser_SendHmiStatus();
      }
      else
        if (keyword.compareTo("BUZ") == 0)
        {
          CheckBuzCommand();
        }
        else
          if (keyword.compareTo("KEY") == 0)
          {
            CheckKeyCommand();
          }
          else
            if (keyword.compareTo("DSP") == 0)
            {
              CheckDisplayCommand();
            }
            else          
            {
              Serial1.write("unbekanntes Steuerwort\n");
            }
            ser_ClearInputString();
  }
}

static void CheckDisplayCommand(void)
{
  String  command;
  String  wert;
  uint8_t siz  = (uint8_t)0;
  uint8_t xpos = (uint8_t)0;
  uint8_t ypos = (uint8_t)0;
  uint8_t x0   = (uint8_t)0;
  uint8_t y0   = (uint8_t)0;
  uint8_t x1   = (uint8_t)0;
  uint8_t y1   = (uint8_t)0;
  
  command = inputString.substring(4,8);
  if (command.compareTo("LINE") == 0)
  {
    wert = inputString.substring(9, 11);
    x0 = (uint8_t)wert.toInt();
    wert = inputString.substring(12, 14);
    y0 = (uint8_t)wert.toInt();
    wert = inputString.substring(15, 17);
    x1 = (uint8_t)wert.toInt();
    wert = inputString.substring(18, 20);
    y1 = (uint8_t)wert.toInt();
    display_DrawLine(x0, y0, x1, y1);
    Serial1.write("Ok\n");
  }
  else
  {
    command = inputString.substring(4,8);
    if (command.compareTo("TEXT") == 0)
    {
      command = outputString.substring(9,23);
      display_WriteText(command);
      Serial1.write("Ok\n");
    }
    else
    {
      command = inputString.substring(4,9);
      if (command.compareTo("CLEAN") == 0)
      {
        display_Clear();
        Serial1.write("Ok\n");
      }
      else
      {
        command = inputString.substring(4,8);
        if (command.compareTo("SIZE") == 0)
        {
          wert = inputString.substring(9, 10);
          siz  = (uint8_t)wert.toInt();
          if ((siz < 1) || (siz > 8))
          {
            Serial1.write("Textgroesse wird nicht unterstuetzt\n");
          }
          else
          {
            display_TextSize(siz);
            Serial1.write("Ok\n");
          }
        }
        else
        {
          command = inputString.substring(4,8);
          if (command.compareTo("CURS") == 0)
          {
            wert = inputString.substring(9, 11);
            xpos = (uint8_t)wert.toInt();
            wert = inputString.substring(12, 14);
            ypos = (uint8_t)wert.toInt();
            if ((xpos > DSP_MAXWIDTH) || (ypos > DSP_MAXHEIGHT))
            {
              Serial1.write("Cursorposition ausserhalb\n");
            }
            else
            {
              display_CursorPosition(xpos, ypos);
              Serial1.write("Ok\n");
            }
          } 
          else          
            {
              Serial1.write("unbekannter Display Modus\n");
            }
 
        }
      }
    }
  }
}

static void CheckKeyCommand(void)
{
  String  command;
  
  command = inputString.substring(4,8);
  if (command.compareTo("PUSH") == 0)
  {
    buzkeys_SetPushKey();
    Serial1.write("Ok\n");
  }
  else
  {
    command = inputString.substring(4,8);
    if (command.compareTo("PULL") == 0)
    {
      buzkeys_ClearPushKey();
      Serial1.write("Ok\n");
    }
    else
    {
      command = inputString.substring(4,7);
      if (command.compareTo("GET") == 0)
      {
        buzkeys_CheckKeys();
      }
    }
  }
}

static void CheckBuzCommand(void)
{
  String  command;
  String  wert;
  uint8_t count = (uint8_t)0;
  uint8_t strt  = (uint8_t)0;
  uint8_t wrt   = (uint8_t)0;
  
  command = inputString.substring(4,6);
  if (command.compareTo("AN") == 0)
  {
    buzkeys_SetBuzz();
    Serial1.write("Ok\n");
  }
  else
  {
    command = inputString.substring(4,7);
    if (command.compareTo("AUS") == 0)
    {
      buzkeys_ClearBuzz();
      Serial1.write("Ok\n");
    }
    else
    {
      command = inputString.substring(4,12);
      if (command.compareTo("INTERVAL") == 0)
      {
        wert = inputString.substring(13, 17);
        wrt = wert.toInt();
        buzkeys_SetPeriod(wrt);
        strt = CheckPosition(wrt);
        wert = inputString.substring(strt + 14, strt + 17);
        count = wert.toInt();
        buzkeys_SetIntervallCount(count);
        if (count <= 100)
        {
          Serial1.write("Ok\n");
        }
        else
        {
          Serial1.write("Intervallzähler zu gross\n");
        }
      }
    }
  }
}

static uint8_t CheckPosition(uint32_t vl)
{
  uint8_t retval = (uint8_t)0;
  
  if (vl < 10)
  {
    retval = 1;
  }
  if ((vl >= 10) && (vl < 100))
  {
    retval = 2;
  }
  if((vl >= 100) && (vl < 1000))
  {
    retval = 3;
  }
  if ((vl >= 1000) && (vl < 10000))
  {
    retval = 4;
  }
  if ((vl >= 10000) && (vl < 100000))
  {
    retval = 5;
  }
  return(retval);
}

static void CheckLedCommand(void)
{
  String color;

  color = inputString.substring(4,7);
  if (color.compareTo("ROT") == 0)
  {
    CheckLedSubCommand(red, 8);
  }
  else
  {
    color = inputString.substring(4,8);
    if (color.compareTo("BLAU") == 0)
    {
      CheckLedSubCommand(blue, 9);
    }
    else
    {
      color = inputString.substring(4,9);
      if (color.compareTo("GRUEN") == 0)
      {
        CheckLedSubCommand(green, 10);
      }
      else
      {
        color = inputString.substring(4,8);
        if (color.compareTo("BACK") == 0)
        {
          CheckLedSubCommand(backlight, 9);
        }
        else
        {
          Serial1.write("unbekannter LED Typ\n");
        }
      }
    }
  }
}

static void CheckLedSubCommand(ledcolor led, uint8_t startpos)
{
  String command;
  String wert;
  long   blinkwert = 0;
  long   dimmwert = 0;

  command = inputString.substring(startpos,startpos + 2);
  if (command.compareTo("AN") == 0)
  {
    switch(led)
    {
      case red:
        leds_SetRed();
        break;
      case green:
        leds_SetGreen();
        break;
      case blue:
        leds_SetBlue();
        break;
      case backlight:
        leds_SetBacklight();
        break;
    }
    Serial1.write("Ok\n");
  }
  else
  {
    command = inputString.substring(startpos,startpos + 3);
    if (command.compareTo("AUS") == 0)
    {
      switch(led)
      {
        case red:
          leds_ClearRed();
          break;
        case green:
          leds_ClearGreen();
          break;
        case blue:
          leds_ClearBlue();
          break;
        case backlight:
          leds_ClearBacklight();
          break;
      }
      Serial1.write("Ok\n");
    }
    else
    {
      command = inputString.substring(startpos,startpos + 5);
      if (command.compareTo("BLINK") == 0)
      {
        wert = inputString.substring(startpos + 5, startpos + 10);
        blinkwert = wert.toInt();
        if (blinkwert >= 100)
        {
          switch(led)
          {
            case red:
              leds_SetBlinkRed((unsigned long)blinkwert);
              break;
            case green:
              leds_SetBlinkGreen((unsigned long)blinkwert);
              break;
            case blue:
              leds_SetBlinkBlue((unsigned long)blinkwert);
              break;
            case backlight:
              leds_SetBlinkBacklight((unsigned long)blinkwert);
              break;
          }
          Serial1.write("Ok\n");
        }
        else
        {
          Serial1.write("LED Blinkwert zu klein\n");
        }
      }
      else
      {
        command = inputString.substring(startpos,startpos + 4);
        if (command.compareTo("DIMM") == 0)
        {
          wert = inputString.substring(startpos + 5, startpos + 10);
          dimmwert = wert.toInt();
          if ((dimmwert <= 255) && (dimmwert >= 0))
          {
            switch(led)
            {
              case red:
                leds_DimmRed((int)dimmwert);
                break;
              case green:
                leds_DimmGreen((int)dimmwert);
                break;
              case blue:
                leds_DimmBlue((int)dimmwert);
                break;
              case backlight:
                leds_DimmBacklight((int)dimmwert);
                break;
            }
            Serial1.write("Ok\n");
          }
          else
          {
            Serial1.write("Dimmwert zu gross\n");
          }
        }
        else
        {
          Serial1.write("unbekannter LED Modus\n");
        }
      }
    }
  }
}

/*
 * sendet den Geraetestatus
 */
static void ser_SendHmiStatus(void)
{
  Serial1.write("Keys: 6\n");
  Serial1.write("Led's: Rot, Gruen, Blau, Back\n");  
  Serial1.write("Sound: Buzzer\n");
  Serial1.write("Display: 84Spalten * 48Zeilen\n");
}

/*
 * callback fuer empfangene Zeichen von der seriellen Schnittstelle
 */
void serialEvent1(void)
{
  char inChar;
  
  while (Serial1.available()) 
  {
    inChar = (char)Serial1.read();
    inputString += inChar;
    if (inChar == '\n') 
    {
      stringComplete = true;
    }
  }
}
